diff mbox series

[v5,2/3] vdpa: Restore receive-side scaling state

Message ID cf5b78a16ed0318982ceffb195f2227f6aad4ac1.1698195059.git.yin31149@gmail.com
State New
Headers show
Series Vhost-vdpa Shadow Virtqueue RSS Support | expand

Commit Message

Hawkins Jiawei Oct. 25, 2023, 1:08 a.m. UTC
This patch reuses vhost_vdpa_net_load_rss() with some
refactorings to restore the receive-side scaling state
at device's startup.

Signed-off-by: Hawkins Jiawei <yin31149@gmail.com>
---
v5:
  - resolve conflict with the updated patch 
"Vhost-vdpa Shadow Virtqueue Hash calculation Support"

v4: https://lore.kernel.org/all/79caf9bf05778ed5279e11bdd1f26b49baf373ce.1697904740.git.yin31149@gmail.com/
  - add do_rss argument and relative code in vhost_vdpa_net_load_rss()

v3: https://lore.kernel.org/all/47b17e160ba4e55b24790b7d73b22d2b437ebe3c.1693299194.git.yin31149@gmail.com/
  - resolve conflict with updated patch
"Vhost-vdpa Shadow Virtqueue Hash calculation Support"

RFC v2: https://lore.kernel.org/all/af33aa80bc4ef0b2cec6c21b9448866c517fde80.1691926415.git.yin31149@gmail.com/
  - Correct the feature usage to VIRTIO_NET_F_HASH_REPORT when
loading the hash calculation state

RFC v1: https://lore.kernel.org/all/93d5d82f0a5df71df326830033e50358c8b6be7a.1691766252.git.yin31149@gmail.com/

 net/vhost-vdpa.c | 67 +++++++++++++++++++++++++++++++-----------------
 1 file changed, 44 insertions(+), 23 deletions(-)

Comments

Eugenio Perez Martin Nov. 3, 2023, 2:42 p.m. UTC | #1
On Wed, Oct 25, 2023 at 3:08 AM Hawkins Jiawei <yin31149@gmail.com> wrote:
>
> This patch reuses vhost_vdpa_net_load_rss() with some
> refactorings to restore the receive-side scaling state
> at device's startup.
>

Acked-by: Eugenio Pérez <eperezma@redhat.com>

> Signed-off-by: Hawkins Jiawei <yin31149@gmail.com>
> ---
> v5:
>   - resolve conflict with the updated patch
> "Vhost-vdpa Shadow Virtqueue Hash calculation Support"
>
> v4: https://lore.kernel.org/all/79caf9bf05778ed5279e11bdd1f26b49baf373ce.1697904740.git.yin31149@gmail.com/
>   - add do_rss argument and relative code in vhost_vdpa_net_load_rss()
>
> v3: https://lore.kernel.org/all/47b17e160ba4e55b24790b7d73b22d2b437ebe3c.1693299194.git.yin31149@gmail.com/
>   - resolve conflict with updated patch
> "Vhost-vdpa Shadow Virtqueue Hash calculation Support"
>
> RFC v2: https://lore.kernel.org/all/af33aa80bc4ef0b2cec6c21b9448866c517fde80.1691926415.git.yin31149@gmail.com/
>   - Correct the feature usage to VIRTIO_NET_F_HASH_REPORT when
> loading the hash calculation state
>
> RFC v1: https://lore.kernel.org/all/93d5d82f0a5df71df326830033e50358c8b6be7a.1691766252.git.yin31149@gmail.com/
>
>  net/vhost-vdpa.c | 67 +++++++++++++++++++++++++++++++-----------------
>  1 file changed, 44 insertions(+), 23 deletions(-)
>
> diff --git a/net/vhost-vdpa.c b/net/vhost-vdpa.c
> index 3466936b87..a4cc1381fc 100644
> --- a/net/vhost-vdpa.c
> +++ b/net/vhost-vdpa.c
> @@ -828,7 +828,7 @@ static int vhost_vdpa_net_load_mac(VhostVDPAState *s, const VirtIONet *n,
>
>  static int vhost_vdpa_net_load_rss(VhostVDPAState *s, const VirtIONet *n,
>                                     struct iovec *out_cursor,
> -                                   struct iovec *in_cursor)
> +                                   struct iovec *in_cursor, bool do_rss)
>  {
>      struct virtio_net_rss_config cfg = {};
>      ssize_t r;
> @@ -854,21 +854,35 @@ static int vhost_vdpa_net_load_rss(VhostVDPAState *s, const VirtIONet *n,
>                         sizeof(n->rss_data.indirections_table[0]));
>      cfg.hash_types = cpu_to_le32(n->rss_data.hash_types);
>
> -    /*
> -     * According to VirtIO standard, "Field reserved MUST contain zeroes.
> -     * It is defined to make the structure to match the layout of
> -     * virtio_net_rss_config structure, defined in 5.1.6.5.7.".
> -     *
> -     * Therefore, we need to zero the fields in
> -     * struct virtio_net_rss_config, which corresponds to the
> -     * `reserved` field in struct virtio_net_hash_config.
> -     *
> -     * Note that all other fields are zeroed at their definitions,
> -     * except for the `indirection_table` field, where the actual data
> -     * is stored in the `table` variable to ensure compatibility
> -     * with RSS case. Therefore, we need to zero the `table` variable here.
> -     */
> -    table[0] = 0;
> +    if (do_rss) {
> +        /*
> +         * According to VirtIO standard, "Number of entries in indirection_table
> +         * is (indirection_table_mask + 1)".
> +         */
> +        cfg.indirection_table_mask = cpu_to_le16(n->rss_data.indirections_len -
> +                                                 1);
> +        cfg.unclassified_queue = cpu_to_le16(n->rss_data.default_queue);
> +        for (int i = 0; i < n->rss_data.indirections_len; ++i) {
> +            table[i] = cpu_to_le16(n->rss_data.indirections_table[i]);
> +        }
> +        cfg.max_tx_vq = cpu_to_le16(n->curr_queue_pairs);
> +    } else {
> +        /*
> +         * According to VirtIO standard, "Field reserved MUST contain zeroes.
> +         * It is defined to make the structure to match the layout of
> +         * virtio_net_rss_config structure, defined in 5.1.6.5.7.".
> +         *
> +         * Therefore, we need to zero the fields in
> +         * struct virtio_net_rss_config, which corresponds to the
> +         * `reserved` field in struct virtio_net_hash_config.
> +         *
> +         * Note that all other fields are zeroed at their definitions,
> +         * except for the `indirection_table` field, where the actual data
> +         * is stored in the `table` variable to ensure compatibility
> +         * with RSS case. Therefore, we need to zero the `table` variable here.
> +         */
> +        table[0] = 0;
> +    }
>
>      /*
>       * Considering that virtio_net_handle_rss() currently does not restore
> @@ -899,6 +913,7 @@ static int vhost_vdpa_net_load_rss(VhostVDPAState *s, const VirtIONet *n,
>
>      r = vhost_vdpa_net_load_cmd(s, out_cursor, in_cursor,
>                                  VIRTIO_NET_CTRL_MQ,
> +                                do_rss ? VIRTIO_NET_CTRL_MQ_RSS_CONFIG :
>                                  VIRTIO_NET_CTRL_MQ_HASH_CONFIG,
>                                  data, ARRAY_SIZE(data));
>      if (unlikely(r < 0)) {
> @@ -933,13 +948,19 @@ static int vhost_vdpa_net_load_mq(VhostVDPAState *s,
>          return r;
>      }
>
> -    if (!virtio_vdev_has_feature(&n->parent_obj, VIRTIO_NET_F_HASH_REPORT)) {
> -        return 0;
> -    }
> -
> -    r = vhost_vdpa_net_load_rss(s, n, out_cursor, in_cursor);
> -    if (unlikely(r < 0)) {
> -        return r;
> +    if (virtio_vdev_has_feature(&n->parent_obj, VIRTIO_NET_F_RSS)) {
> +        /* load the receive-side scaling state */
> +        r = vhost_vdpa_net_load_rss(s, n, out_cursor, in_cursor, true);
> +        if (unlikely(r < 0)) {
> +            return r;
> +        }
> +    } else if (virtio_vdev_has_feature(&n->parent_obj,
> +                                       VIRTIO_NET_F_HASH_REPORT)) {
> +        /* load the hash calculation state */
> +        r = vhost_vdpa_net_load_rss(s, n, out_cursor, in_cursor, false);
> +        if (unlikely(r < 0)) {
> +            return r;
> +        }
>      }
>
>      return 0;
> --
> 2.25.1
>
diff mbox series

Patch

diff --git a/net/vhost-vdpa.c b/net/vhost-vdpa.c
index 3466936b87..a4cc1381fc 100644
--- a/net/vhost-vdpa.c
+++ b/net/vhost-vdpa.c
@@ -828,7 +828,7 @@  static int vhost_vdpa_net_load_mac(VhostVDPAState *s, const VirtIONet *n,
 
 static int vhost_vdpa_net_load_rss(VhostVDPAState *s, const VirtIONet *n,
                                    struct iovec *out_cursor,
-                                   struct iovec *in_cursor)
+                                   struct iovec *in_cursor, bool do_rss)
 {
     struct virtio_net_rss_config cfg = {};
     ssize_t r;
@@ -854,21 +854,35 @@  static int vhost_vdpa_net_load_rss(VhostVDPAState *s, const VirtIONet *n,
                        sizeof(n->rss_data.indirections_table[0]));
     cfg.hash_types = cpu_to_le32(n->rss_data.hash_types);
 
-    /*
-     * According to VirtIO standard, "Field reserved MUST contain zeroes.
-     * It is defined to make the structure to match the layout of
-     * virtio_net_rss_config structure, defined in 5.1.6.5.7.".
-     *
-     * Therefore, we need to zero the fields in
-     * struct virtio_net_rss_config, which corresponds to the
-     * `reserved` field in struct virtio_net_hash_config.
-     *
-     * Note that all other fields are zeroed at their definitions,
-     * except for the `indirection_table` field, where the actual data
-     * is stored in the `table` variable to ensure compatibility
-     * with RSS case. Therefore, we need to zero the `table` variable here.
-     */
-    table[0] = 0;
+    if (do_rss) {
+        /*
+         * According to VirtIO standard, "Number of entries in indirection_table
+         * is (indirection_table_mask + 1)".
+         */
+        cfg.indirection_table_mask = cpu_to_le16(n->rss_data.indirections_len -
+                                                 1);
+        cfg.unclassified_queue = cpu_to_le16(n->rss_data.default_queue);
+        for (int i = 0; i < n->rss_data.indirections_len; ++i) {
+            table[i] = cpu_to_le16(n->rss_data.indirections_table[i]);
+        }
+        cfg.max_tx_vq = cpu_to_le16(n->curr_queue_pairs);
+    } else {
+        /*
+         * According to VirtIO standard, "Field reserved MUST contain zeroes.
+         * It is defined to make the structure to match the layout of
+         * virtio_net_rss_config structure, defined in 5.1.6.5.7.".
+         *
+         * Therefore, we need to zero the fields in
+         * struct virtio_net_rss_config, which corresponds to the
+         * `reserved` field in struct virtio_net_hash_config.
+         *
+         * Note that all other fields are zeroed at their definitions,
+         * except for the `indirection_table` field, where the actual data
+         * is stored in the `table` variable to ensure compatibility
+         * with RSS case. Therefore, we need to zero the `table` variable here.
+         */
+        table[0] = 0;
+    }
 
     /*
      * Considering that virtio_net_handle_rss() currently does not restore
@@ -899,6 +913,7 @@  static int vhost_vdpa_net_load_rss(VhostVDPAState *s, const VirtIONet *n,
 
     r = vhost_vdpa_net_load_cmd(s, out_cursor, in_cursor,
                                 VIRTIO_NET_CTRL_MQ,
+                                do_rss ? VIRTIO_NET_CTRL_MQ_RSS_CONFIG :
                                 VIRTIO_NET_CTRL_MQ_HASH_CONFIG,
                                 data, ARRAY_SIZE(data));
     if (unlikely(r < 0)) {
@@ -933,13 +948,19 @@  static int vhost_vdpa_net_load_mq(VhostVDPAState *s,
         return r;
     }
 
-    if (!virtio_vdev_has_feature(&n->parent_obj, VIRTIO_NET_F_HASH_REPORT)) {
-        return 0;
-    }
-
-    r = vhost_vdpa_net_load_rss(s, n, out_cursor, in_cursor);
-    if (unlikely(r < 0)) {
-        return r;
+    if (virtio_vdev_has_feature(&n->parent_obj, VIRTIO_NET_F_RSS)) {
+        /* load the receive-side scaling state */
+        r = vhost_vdpa_net_load_rss(s, n, out_cursor, in_cursor, true);
+        if (unlikely(r < 0)) {
+            return r;
+        }
+    } else if (virtio_vdev_has_feature(&n->parent_obj,
+                                       VIRTIO_NET_F_HASH_REPORT)) {
+        /* load the hash calculation state */
+        r = vhost_vdpa_net_load_rss(s, n, out_cursor, in_cursor, false);
+        if (unlikely(r < 0)) {
+            return r;
+        }
     }
 
     return 0;